#!/usr/bin/python3

def escape(str):
	result = ""
	for c in str:
		if c=='"':
			result += '\\"'
		elif c=='\\':
			result += '\\\\'
		elif ord(c)<0x20:
			result += "\\{:03o}".format(ord(c))
		else:
			result += c
	return result

def make_tex(fn, mipmap=False, srgb=False):
	from PIL import Image

	img = Image.open(fn)

	fmt = "".join(img.getbands())
	if fmt=="LA":
		fmt = "LUMINANCE8_ALPHA8"
	elif fmt=="L":
		fmt = "LUMINANCE8"

	if srgb:
		fmt = "S"+fmt

	result = "storage {} {} {};\n".format(fmt, img.size[0], img.size[1])
	if mipmap:
		result += "generate_mipmap true;\n"
	result += "raw_data \""
	data = list(img.getdata())
	for y in range(img.size[1]):
		i = (img.size[1]-1-y)*img.size[0]
		if len(img.getbands())==1:
			result += escape("".join([chr(v) for v in data[i:i+img.size[0]]]))
		else:
			result += escape("".join(["".join([chr(v) for v in p]) for p in data[i:i+img.size[0]]]))
	result += "\";\n"

	return result

if __name__=="__main__":
	import os
	import argparse

	parser = argparse.ArgumentParser()
	parser.add_argument("-o", "--output", metavar="FILE", help="Output filename")
	parser.add_argument("-m", "--mipmap", action="store_true", help="Enable mipmaps")
	parser.add_argument("--srgb", action="store_const", const=True, help="Use sRGB color space")
	parser.add_argument("image")

	args = parser.parse_args()

	out_fn = args.output
	if not out_fn:
		out_fn = os.path.splitext(args.image)[0]+".tex2d"
	out = open(out_fn, "w")
	out.write(make_tex(args.image, args.mipmap, args.srgb))
