#include <stdexcept>
#include <msp/gl/extensions/arb_shader_objects.h>
#include <msp/gl/extensions/arb_uniform_buffer_object.h>
#include "buffer.h"
#include "color.h"
#include "error.h"
#include "matrix.h"
#include "uniform.h"
#include "uniformblock.h"
#include "vector.h"

using namespace std;

namespace Msp {
namespace GL {

DefaultUniformBlock::DefaultUniformBlock()
{
	static Require _req(ARB_shader_objects);
}

void DefaultUniformBlock::attach(const Program::UniformInfo &info, const Uniform &uni)
{
	if(info.block->bind_point>=0)
		throw invalid_argument("DefaultUniformBlock::attach");

	attach(info.location, uni);
}

void DefaultUniformBlock::attach(int index, const Uniform &uni)
{
	if(index<0)
		return;

	if(static_cast<unsigned>(index)>=uniforms.size())
		uniforms.resize(index+1, 0);
	uniforms[index] = &uni;
}

void DefaultUniformBlock::apply(int index) const
{
	if(index>=0)
		throw invalid_argument("DefaultUniformBlock::apply");

	for(unsigned i=0; i<uniforms.size(); ++i)
		if(uniforms[i])
			uniforms[i]->apply(i);
}


BufferBackedUniformBlock::BufferBackedUniformBlock(unsigned s):
	size(s),
	data(size),
	buf_range(0)
{
	static Require _req(ARB_shader_objects);
	static Require _req2(ARB_uniform_buffer_object);

	if(!size)
		throw invalid_argument("BufferBackedUniformBlock::BufferBackedUniformBlock");
}

BufferBackedUniformBlock::~BufferBackedUniformBlock()
{
	delete buf_range;
}

unsigned BufferBackedUniformBlock::get_alignment() const
{
	return BufferRange::get_uniform_buffer_alignment();
}

void BufferBackedUniformBlock::location_changed(Buffer *buf, unsigned off, unsigned) const
{
	delete buf_range;
	buf_range = buf->create_range(off, size);
}

void BufferBackedUniformBlock::attach(const Program::UniformInfo &info, const Uniform &uni)
{
	if(info.block->bind_point<0)
		throw invalid_argument("BufferBackedUniformBlock::attach");

	uni.store(info, &data[info.offset]);
	dirty = true;
}

void BufferBackedUniformBlock::apply(int index) const
{
	if(index<0)
		throw invalid_argument("BufferBackedUniformBlock::apply");
	if(!get_buffer())
		throw invalid_operation("UniformBlock::apply");

	refresh();
	buf_range->bind_to(UNIFORM_BUFFER, index);
}

} // namespace GL
} // namespace Msp
