#ifndef MSP_GL_RENDERPASS_H_
#define MSP_GL_RENDERPASS_H_

#include <msp/core/refptr.h>
#include <msp/datafile/objectloader.h>
#include "material.h"

namespace Msp {
namespace GL {

class Program;
class ProgramData;
class Renderer;
class Sampler;
class Texture;
class Texturing;

/**
Encapsulates the data that determines the appearance of a rendered surface.
This includes shader and data for it, material and texturing.
*/
class RenderPass
{
public:
	class Loader: public DataFile::CollectionObjectLoader<RenderPass>
	{
	public:
		Loader(RenderPass &);
		Loader(RenderPass &, Collection &);

	private:
		void init();

		static std::string get_shader_name(const std::string &);

		void material_inline();
		void material(const std::string &);
		void shader(const std::string &);
		void texture(const std::string &);
		void texunit(unsigned);
		void texunit_named(unsigned, const std::string &);
		void uniforms();
		void uniform_slot(const std::string &);
		void uniform_slot2(const std::string &, const std::string &);
	};

private:
	struct TextureSlot
	{
		class Loader: public DataFile::CollectionObjectLoader<TextureSlot>
		{
		private:
			std::string auto_slot_name;

		public:
			Loader(TextureSlot &, const std::string &, Collection *);

		private:
			void slot_auto();
		};

		Tag tag;
		std::string slot_name;
		const Texture *texture;
		const Sampler *sampler;

		TextureSlot(Tag t): tag(t), texture(0), sampler(0) { }
	};

	RefPtr<const Program> shprog;
	bool shprog_from_material;
	RefPtr<ProgramData> shdata;
	std::map<Tag, Tag> uniform_slots;
	RefPtr<const Material> material;
	std::string material_slot;
	std::vector<TextureSlot> textures;
	bool back_faces;

public:
	RenderPass();

private:
	void finalize_material(DataFile::Collection *);
	void maybe_create_material_shader(DataFile::Collection *);
	void ensure_private_shader_data();

public:
	void set_shader_program(const Program *, const ProgramData *);
	const Program *get_shader_program() const { return shprog.get(); }
	const ProgramData *get_shader_data() const { return shdata.get(); }
	Tag get_slotted_uniform_tag(Tag) const;
	void set_material(const Material *, DataFile::Collection * = 0);
	const Material *get_material() const { return material.get(); }
	const std::string &get_material_slot_name() const { return material_slot; }
	void set_texture(Tag, const Texture *, const Sampler * = 0);
	Tag get_texture_tag(const std::string &) const;
	DEPRECATED void set_texture(unsigned, const Texture *, const Sampler * = 0);
	DEPRECATED const Texturing *get_texturing() const { return 0; }
	DEPRECATED int get_texture_index(const std::string &) const;
	void set_back_faces(bool);
	bool get_back_faces() const { return back_faces; }

	void apply(Renderer &) const;
};

} // namespace GL
} // namespace Msp

#endif
